@@ -1,5 +1,5 @@
 package Cpanel::JSON::XS;
-our $VERSION = '3.0211';
+our $VERSION = '3.0212';
 
 =pod
 
@@ -992,7 +992,7 @@ This is useful if your JSON texts are not delimited by an outer protocol
 and you need to know where the JSON text ends.
 
    Cpanel::JSON::XS->new->decode_prefix ("[1] the tail")
-   => ([], 3)
+   => ([1], 3)
 
 =item $json->to_json ($perl_hash_or_arrayref)
 
@@ -1838,6 +1838,23 @@ For updated graphs see L<https://github.com/Sereal/Sereal/wiki/Sereal-Comparison
 
 =head1 INTEROP with JSON and JSON::XS and other JSON modules
 
+As long as you only serialize data that can be directly expressed in
+JSON, C<Cpanel::JSON::XS> is incapable of generating invalid JSON
+output (modulo bugs, but C<JSON::XS> has found more bugs in the
+official JSON testsuite (1) than the official JSON testsuite has found
+in C<JSON::XS> (0)).
+
+When you have trouble decoding JSON generated by this module using other
+decoders, then it is very likely that you have an encoding mismatch or the
+other decoder is broken.
+
+When decoding, C<JSON::XS> is strict by default and will likely catch
+all errors. There are currently two settings that change this:
+C<relaxed> makes C<JSON::XS> accept (but not generate) some
+non-standard extensions, and C<allow_tags> or C<allow_blessed> will
+allow you to encode and decode Perl objects, at the cost of being
+totally insecure and not outputting valid JSON anymore.
+
 JSON-XS-3.01 broke interoperability with JSON-2.90 with booleans. See L<JSON>.
 
 Cpanel::JSON::XS needs to know the JSON and JSON::XS versions to be able work
@@ -1846,19 +1863,83 @@ So you need to load these modules before.
 
 true/false overloading and boolean representations are supported.
 
-JSON::XS and JSON::PP representations are accepted and older JSON::XS accepts
-Cpanel::JSON::XS booleans. All JSON modules JSON, JSON, PP, JSON::XS, Cpanel::JSON::XS
-produce JSON::PP::Boolean objects, just Mojo and JSON::YAJL not.
-Mojo produces Mojo::JSON::_Bool and JSON::YAJL::Parser just an unblessed IV.
+JSON::XS and JSON::PP representations are accepted and older JSON::XS
+accepts Cpanel::JSON::XS booleans. All JSON modules JSON, JSON, PP,
+JSON::XS, Cpanel::JSON::XS produce JSON::PP::Boolean objects, just
+Mojo and JSON::YAJL not.  Mojo produces Mojo::JSON::_Bool and
+JSON::YAJL::Parser just an unblessed IV.
 
 Cpanel::JSON::XS accepts JSON::PP::Boolean and Mojo::JSON::_Bool objects as booleans.
 
 I cannot think of any reason to still use JSON::XS anymore.
 
+
+=head2 TAGGED VALUE SYNTAX AND STANDARD JSON EN/DECODERS
+
+When you use C<allow_tags> to use the extended (and also nonstandard
+and invalid) JSON syntax for serialized objects, and you still want to
+decode the generated serialize objects, you can run a regex to replace
+the tagged syntax by standard JSON arrays (it only works for "normal"
+package names without comma, newlines or single colons). First, the
+readable Perl version:
+
+   # if your FREEZE methods return no values, you need this replace first:
+   $json =~ s/\( \s* (" (?: [^\\":,]+|\\.|::)* ") \s* \) \s* \[\s*\]/[$1]/gx;
+
+   # this works for non-empty constructor arg lists:
+   $json =~ s/\( \s* (" (?: [^\\":,]+|\\.|::)* ") \s* \) \s* \[/[$1,/gx;
+
+And here is a less readable version that is easy to adapt to other
+languages:
+
+   $json =~ s/\(\s*("([^\\":,]+|\\.|::)*")\s*\)\s*\[/[$1,/g;
+
+Here is an ECMAScript version (same regex):
+
+   json = json.replace (/\(\s*("([^\\":,]+|\\.|::)*")\s*\)\s*\[/g, "[$1,");
+
+Since this syntax converts to standard JSON arrays, it might be hard to
+distinguish serialized objects from normal arrays. You can prepend a
+"magic number" as first array element to reduce chances of a collision:
+
+   $json =~ s/\(\s*("([^\\":,]+|\\.|::)*")\s*\)\s*\[/["XU1peReLzT4ggEllLanBYq4G9VzliwKF",$1,/g;
+
+And after decoding the JSON text, you could walk the data
+structure looking for arrays with a first element of
+C<XU1peReLzT4ggEllLanBYq4G9VzliwKF>.
+
+The same approach can be used to create the tagged format with another
+encoder. First, you create an array with the magic string as first member,
+the classname as second, and constructor arguments last, encode it as part
+of your JSON structure, and then:
+
+   $json =~ s/\[\s*"XU1peReLzT4ggEllLanBYq4G9VzliwKF"\s*,\s*("([^\\":,]+|\\.|::)*")\s*,/($1)[/g;
+
+Again, this has some limitations - the magic string must not be encoded
+with character escapes, and the constructor arguments must be non-empty.
+
+
+=head1 RFC7159
+
+Since this module was written, Google has written a new JSON RFC, RFC 7159
+(and RFC7158). Unfortunately, this RFC breaks compatibility with both the
+original JSON specification on www.json.org and RFC4627.
+
+As far as I can see, you can get partial compatibility when parsing by
+using C<< ->allow_nonref >>. However, consider the security implications
+of doing so.
+
+I haven't decided yet when to break compatibility with RFC4627 by default
+(and potentially leave applications insecure) and change the default to
+follow RFC7159, but application authors are well advised to call C<<
+->allow_nonref(0) >> even if this is the current default, if they cannot
+handle non-reference values, in preparation for the day when the default
+will change.
+
 =head1 SECURITY CONSIDERATIONS
 
-JSON::XS is not only fast, JSON is generally the most secure
-serializing format, because it is the only one besides
+JSON::XS and Cpanel::JSON::XS are not only fast, JSON is generally the
+most secure serializing format, because it is the only one besides
 Data::MessagePack, which does not deserialize objects per default. For
 all languages, not just perl.  The binary variant BSON (MongoDB) does
 more but is unsafe.
@@ -1929,6 +2010,46 @@ it, as major browser developers care only for features, not about getting
 security right). You might also want to also look at L<Mojo::JSON>
 special escape rules to prevent from XSS attacks.
 
+=head1 "OLD" VS. "NEW" JSON (RFC 4627 VS. RFC 7159)
+
+TL;DR: Due to security concerns, Cpanel::JSON::XS will not allow
+scalar data in JSON texts by default - you need to create your own
+Cpanel::JSON::XS object and enable C<allow_nonref>:
+
+
+   my $json = JSON::XS->new->allow_nonref;
+
+   $text = $json->encode ($data);
+   $data = $json->decode ($text);
+
+The long version: JSON being an important and supposedly stable format,
+the IETF standardized it as RFC 4627 in 2006. Unfortunately the inventor
+of JSON Douglas Crockford unilaterally changed the definition of JSON in
+javascript. Rather than create a fork, the IETF decided to standardize the
+new syntax (apparently, so I as told, without finding it very amusing).
+
+The biggest difference between the original JSON and the new JSON is that
+the new JSON supports scalars (anything other than arrays and objects) at
+the top-level of a JSON text. While this is strictly backwards compatible
+to older versions, it breaks a number of protocols that relied on sending
+JSON back-to-back, and is a minor security concern.
+
+For example, imagine you have two banks communicating, and on one side,
+the JSON coder gets upgraded. Two messages, such as C<10> and C<1000>
+might then be confused to mean C<101000>, something that couldn't happen
+in the original JSON, because neither of these messages would be valid
+JSON.
+
+If one side accepts these messages, then an upgrade in the coder on either
+side could result in this becoming exploitable.
+
+This module has always allowed these messages as an optional extension, by
+default disabled. The security concerns are the reason why the default is
+still disabled, but future versions might/will likely upgrade to the newer
+RFC as default format, so you are advised to check your implementation
+and/or override the default with C<< ->allow_nonref (0) >> to ensure that
+future versions are safe.
+
 =head1 THREADS
 
 Cpanel::JSON::XS has proper ithreads support, unlike JSON::XS. If you
